#!/usr/bin/env python

##########
# Imports
####

# Python
import sys
import os

# Insights
from redhat_access_insights import constants as insights_constants, parse_config_file
from redhat_access_insights.auto_config import try_auto_configuration

# Coordinator
from insights_coordinator_rhev.constants import RhevCoordinatorConstants
from insights_coordinator_rhev.insights_coordinator_infra.utils.cron import CoordinatorCron as RhevCoordinatorCron
from insights_coordinator_rhev.insights_coordinator_infra.logger import CoordinatorLogger as RhevCoordinatorLogger
from insights_coordinator_rhev.parser import RhevCoordinatorParser
from insights_coordinator_rhev.config import RhevCoordinatorConfig
from insights_coordinator_rhev.utilities import RhevCoordinatorUtilities
from insights_coordinator_rhev.insights_coordinator_infra.utils.language import CoordinatorLanguage as RhevCoordinatorLanguage
from insights_coordinator_rhev.insights_coordinator_infra.transport.ssh import CoordinatorSSH as RhevCoordinatorSSH
from insights_coordinator_rhev.rhevm import RhevCoordinatorRhevm
from insights_coordinator_rhev.insights_coordinator_infra.utils.versions import CoordinatorVersions as RhevCoordinatorVersions


##########################
# Main
####


def main():

    # Setup the Language Handler
    rhev_language = RhevCoordinatorLanguage()

    # Run Coordinator as root
    if os.geteuid() is not 0:
        sys.exit(rhev_language.get_language("Insights Coordinator for RHEV must be run as root"))

    # RHEV Specific constants
    rhev_constants = RhevCoordinatorConstants()

    # RHEV Configuration
    rhev_config = RhevCoordinatorConfig(constants=rhev_constants,
                                        language=rhev_language)

    # Configure Insights Uploader
    uploader_conf = parse_config_file(insights_constants.default_conf_file)
    try_auto_configuration(uploader_conf)

    # Parse options
    options_parser = RhevCoordinatorParser(constants=rhev_constants,
                                           config=rhev_config,
                                           language=rhev_language)
    options = options_parser.parse_options()
    options.uploader_conf = uploader_conf
    if options.interactive is True:
        options.verbose = True

    # Setup the Coordinator Logger
    rhev_logger = RhevCoordinatorLogger(constants=rhev_constants,
                                        config=rhev_config,
                                        language=rhev_language,
                                        verbose=options.verbose)
    rhev_config.setup_logger(rhev_logger)
    options_parser.setup_logger(rhev_logger)

    # Print configuration and options
    rhev_config.print_config()
    options_parser.print_options()

    # Display the current coordinator version and exit if --version flag is thrown
    if options.version is True:
        sys.exit(rhev_constants.version)

    # Exit if a proper uploader configuration file is not found
    # TODO: If a valid uploader_conf was not found BUT --archive-only was thrown as a flag, should we continue?
    # Does this imply that the Insights client is not installed?
    if not uploader_conf:
        sys.exit(rhev_language.get_language("Error configuring Insights Client. Please ensure you have "
                                            "Insights installed and it is up to date."))

    # Schedule the Coordinator Cron Job
    cron = RhevCoordinatorCron(constants=rhev_constants,
                               config=rhev_config,
                               logger=rhev_logger,
                               language=rhev_language)
    rhev_config.setup_cron(cron)

    # Are we running the configuration utility?
    if options.configure is True:
        rhev_config.run_config()
        sys.exit(rhev_language.get_language("RHEV Insights Coordinator Configuration utility completed."))

    # Both enable_cron and disable_cron cant both be flagged or passed in as options
    # If both are passed in then exist with the appropriate message
    if options.enable_cron is True and options.disable_cron is True:
        sys.exit(rhev_language.get_language("Invalid flag combination. You can either enable the Coordinator "
                                            "cron job or disable the Coordinator cron job but not both."))

    # Enable the cron
    if options.enable_cron is True:
        cron.setup_cron()
        rhev_config.write_config_var('enable_cron', True)
        sys.exit(rhev_language.get_language("Cron Job enabled."))
    if rhev_config.get_config_var('enable_cron', True) is True:
        if cron.cron_exists() is False:
            cron.setup_cron()

    # Disable the cron
    if options.disable_cron is True:
        cron.disable_cron()
        rhev_config.write_config_var('enable_cron', False)
        sys.exit(rhev_language.get_language("Cron Job disabled."))
    if rhev_config.get_config_var('enable_cron', True) is False:
        if cron.cron_exists() is True:
            cron.disable_cron()

    # Rhev Utilities
    rhev_utilities = RhevCoordinatorUtilities(constants=rhev_constants,
                                              logger=rhev_logger,
                                              uploader_conf=uploader_conf,
                                              config=rhev_config,
                                              language=rhev_language)
    rhev_config.setup_utilities(rhev_utilities)

    # Setup SSH
    rhev_ssh_client = RhevCoordinatorSSH(constants=rhev_constants,
                                         logger=rhev_logger,
                                         config=rhev_config,
                                         language=rhev_language)

    # Instantiate the RHEV Manager
    rhev_rhevm = RhevCoordinatorRhevm(constants=rhev_constants,
                                      logger=rhev_logger,
                                      ssh_client=rhev_ssh_client,
                                      utilities=rhev_utilities,
                                      config=rhev_config,
                                      language=rhev_language)

    # Instantiate the versions handler
    rhev_versions = RhevCoordinatorVersions(coordinator_constants=rhev_constants,
                                            insights_constants=insights_constants,
                                            language=rhev_language,
                                            logger=rhev_logger,
                                            utilities=rhev_utilities)

    # Import the RHEV Coordinator Module
    platform_module = None
    try:
        import rhev as platform_module
    except ImportError:
        sys.exit(rhev_language.get_language("Error importing libraries. "
                                            "Please try re-installing the insights-coordinator-rhev RPM."))

    if not platform_module:
        sys.exit(rhev_language.get_language("Error importing libraries. "
                                            "Please try re-installing the insights-coordinator-rhev RPM."))

    # Run the Coordinator
    platform_module.run(options=options,
                        constants=rhev_constants,
                        utilities=rhev_utilities,
                        logger=rhev_logger,
                        language=rhev_language,
                        rhevm=rhev_rhevm,
                        versions=rhev_versions)

########
# Main
####
if __name__ == "__main__":
    main()
