#!/usr/bin/env python

import sys

from insights_coordinator_rhev.insights_coordinator_infra.api import CoordinatorApi
from ovirtsdk.api import API


class RhevCoordinatorApi(CoordinatorApi):

    # Constructor
    def __init__(self, **kwargs):
        super(RhevCoordinatorApi, self).__init__(**kwargs)
        self.setup_api()

    # Setup a connection to the RHEVM API
    def setup_api(self):
        try:
            self.api_conn = API(url=self.config.get_config_var('api_url'),
                                username=self.config.get_config_var('api_username'),
                                password=self.config.get_config_var('api_password'),
                                debug=self.constants.api_debug,
                                insecure=self.constants.api_insecure)
        except:
            sys.exit(self.language.get_language("There was an error connecting to the RHEVM API. "
                                                "Please verify you have valid user permissions."))
        if self.api_conn:
            self.logger.vlog(self.language.get_language("Successfully connected to the RHEVM API"))
        else:
            sys.exit(self.language.get_language("There was an error connecting to the RHEVM API. "
                                                "Please verify you have valid user permissions."))
        return self.api_conn

    # Get reference to the current RHEVM API Connection
    def get_api(self):
        if self.api_conn:
            return self.api_conn
        else:
            return self.setup_api()

    # Close the RHEVM API Connection
    def close_api(self):
        if self.api_conn:
            self.api_conn.disconnect()
        self.api_conn = None

    # Get all relevant information from the RHEVM API
    def get_manager_data_from_api(self, options=None):
        manager_information = {}

        if options is None:
            self.logger.vlog(self.language.get_language("Manager options were not passed in... Using defaults..."))
            options = self.constants.api_options

        if options:
            api_conn = self.get_api()
            self.logger.vlog(self.language.get_language("Starting manager data collection from API, "
                                                        "this may take a while..."))
            for option in options:
                self.logger.vlog(self.language.get_language("Obtaining '%s' from RHEV API...") % (option))
                the_option_data = getattr(api_conn, option).list()
                manager_information[option] = self._get_attrs(the_option_data)

            self.close_api()

        return manager_information

    # Get attributes from objects
    def _get_attrs(self, things):
        if things is not None:
            # is it a string or an int/long?
            if isinstance(things, basestring) or isinstance(things, (int, long)) or isinstance(things, bool):
                return str(things)

            # is it actually a list of other things with getters/list functionalities?
            elif isinstance(things, list):
                toreturn = []
                for thing in things:
                    toreturn.append(self._get_attrs(thing))
                return toreturn

            # does it have a "list" attribute?
            # recursive call on list()
            elif hasattr(things, 'list'):
                return self._get_attrs(things.list())

            # does it have getters or listers?
            # call each lister
            # call each getter()
            else:
                toreturn = {}

                # get all listers
                for attr in dir(things):
                    try:
                        if 'list' in dir(getattr(things, attr)):
                            toreturn[attr] = self._get_attrs(getattr(things, attr))
                    except:
                        pass

                # get all getters
                getters = [attr for attr in dir(things) if (attr.startswith('get') and not attr.endswith('_') and not attr in self.constants.api_attribute_exclusion_list)]
                if getters:
                    for getter in getters:
                        try:
                            attrinfo = getattr(things, getter)()
                            toreturn[getter.replace('get_', '')] = self._get_attrs(attrinfo)
                        except:
                            pass

                # return stuff
                return toreturn
        else:
            # base case
            return things
