#!/usr/bin/env python
###############################
# rhev.py Platform Module
####

############
# TODO: Should this whole thing be a class?
# __init__.py should do RhevCoordinator().run()
#####

###########
# Imports
####
import sys
import time
import re


############################
# Run the platform module
# Main execution function
# Called from the main coordinator
#####
def run(**kwargs):
    options = kwargs['options']
    constants = kwargs['constants']
    utilities = kwargs['utilities']
    logger = kwargs['logger']
    language = kwargs['language']
    rhevm = kwargs['rhevm']
    versions = kwargs['versions']

    logger.vlog("Running RHEV Insights Coordinator")
    versions.get_versions()
    starttime = time.time()

    # Test the RHEVM API if flag set
    if options.test_api:
        api_status = rhevm.test_api()
        logger.vlog(language.get_language("API Connection Status: %s") % (api_status))
        sys.exit(language.get_language("We are only testing the RHEV Manager API. The coordinator is now exiting."))

    # Parse the blacklist (if present)
    blacklist = []
    if options.blacklist is not False and options.blacklist != '' and len(options.blacklist) > 0:
        blacklist = options.blacklist.split(',')
        blacklist = blacklist if len(blacklist) > 0 else []
    # Clean the blacklist keywords
    # Replace asterisks with (.*)
    # Create anchors
    if len(blacklist) > 0:
        cleaned_blacklist = []
        for keyword in blacklist:
            cleaned_blacklist.append(('^%s$') % (keyword.replace('*', '(.*)')))
        blacklist = cleaned_blacklist
        logger.vlog('Blacklist parsed: %s' % (blacklist))
    else:
        logger.vlog('No blacklist found.')
    rhevm.setup_blacklist(blacklist)

    # Get all hypervisors/ips
    if options.test_hypervisors:
        logger.set_verbosity(True)
    logger.vlog(language.get_language("Preparing to gather hypervisors from the manager..."))
    hypervisor_list = rhevm.get_hypervisor_list()
    rhevm.print_hypervisor_list(hypervisor_list)
    if options.test_hypervisors:
        sys.exit(language.get_language("We are only testing if the hypervisors are alive and can be remotely accessed. "
                                       "Coordinator is exiting."))

    # Setup Manager Options
    manager_options = constants.api_options
    # TODO: Should probably move this if logic into the rhevm class
    if options.api_options:
        if options.api_options == 'none':
            logger.vlog(language.get_language("We are not collecting any metadata from the RHEV Manager API, "
                                              "'none' was passed in."))
            manager_options = []
        elif options.api_options == 'all':
            logger.vlog(language.get_language("We are collecting all metadata from the RHEV Manager API, 'all' "
                                              "was passed in."))
            manager_options = constants.api_options.split(',')
        else:
            manager_options = options.api_options.split(',')
            logger.vlog(language.get_language("We are collecting specific metadata from the RHEV Manager API..."))
            logger.vlog(manager_options)

    # Collect the specific RHEV API Manager metadata
    rhevm.collect_manager_information(manager_options)

    # Parse the blacklist (if present)
    blacklist = []
    if options.blacklist is not False and options.blacklist != '' and len(options.blacklist) > 0:
        blacklist = options.blacklist.split(',')
        blacklist = blacklist if len(blacklist) > 0 else []
    # Clean the blacklist keywords
    # Replace asterisks with (.*)
    # Create anchors
    if len(blacklist) > 0:
        cleaned_blacklist = []
        for keyword in blacklist:
            cleaned_blacklist.append(('^%s$') % (keyword.replace('*', '(.*)')))
        blacklist = cleaned_blacklist
        logger.vlog('Blacklist parsed: %s' % (blacklist))
    else:
        logger.vlog('No blacklist found.')

    # Run Insights on the manager
    # We run Insights on the manager first so that the Collection Rules are automatically updated.
    # This ensures that each subsequent configuration file that is passed to the individual hypervisors
    # contains updated rules
    rhevm.run_insights_on_manager()

    # FOR ALL HYPERVISORS
    # 1) ssh into each hypervisor/ip and install latest client in /tmp, this will be removed once we have the insights
    #    RPM on the hypervisor image
    # 2) run latest client on each hypervisor in /tmp, will be modified to run native installed insights rpm
    #    once we have it on the images
    # 3) get stdout from client
    # 4) clean /tmp
    stdout = ""
    total_hypervisors = len(hypervisor_list)
    for index, hypervisor in enumerate(hypervisor_list):
        logger.vlog("========================================")
        logger.vlog("Processing hypervisor %s out of %s: %s, %s, %s" % (index + 1, total_hypervisors, hypervisor['id'], hypervisor['ip'], hypervisor['name']))
        hypervisor_blacklisted = hypervisor['blacklisted']
        if hypervisor['is_alive'] and hypervisor['can_ssh'] and hypervisor_blacklisted is False:
            rhevm.get_rhevh().install_hypervisor(hypervisor['ip'], options)
            stdout = rhevm.get_rhevh().run_hypervisor(hypervisor['ip'], hypervisor['id'],
                                                      hypervisor['name'], hypervisor['system_id'])
            utilities.write_archive(stdout)
            rhevm.get_rhevh().clean_hypervisor(hypervisor['ip'])
        else:
            if hypervisor['is_alive'] is False:
                logger.vlog('Cannot execute Insights on Hypervisor %s, %s, %s as it is not alive.'
                            % (hypervisor['id'], hypervisor['ip'], hypervisor['name']))
            if hypervisor['can_ssh'] is False:
                logger.vlog('Cannot execute Insights on Hypervisor %s, %s, %s as the Coordinator does not '
                            'have valid SSH access.' % (hypervisor['id'], hypervisor['ip'], hypervisor['name']))
            if hypervisor_blacklisted is True:
                logger.vlog('Hypervisor %s, %s, %s has been blacklisted via --blacklist or the Coordinator '
                            'configuration files. Insights will not be run for this '
                            'Hypervisor.' % (hypervisor['id'], hypervisor['ip'], hypervisor['name']))
        logger.vlog("Finished processing hypervisor.")
        logger.vlog("========================================")

    # Tar everything up and clean
    utilities.compress_archives()

    # Get execution elapsed time
    thetime = time.time() - starttime

    # Upload to Red Hat if we are not only archiving
    upload_success = False
    logger.vlog('Archive Only: %s' % (options.archive_only))
    if options.archive_only and options.archive_only is True:
        logger.vlog(language.get_language("Archive Only was set. Not uploading archives to Red Hat."))
    else:
        logger.vlog(language.get_language("Archive Only flag was not set. Uploading archives to Red Hat."))
        upload_success = utilities.upload_archives(utilities.get_tmpgz(), thetime)

    # Clean the temporary files if we are not only archiving
    logger.vlog('Upload Success: %s' % (upload_success))
    if (options.archive_only and options.archive_only is True) or upload_success is False:
        logger.vlog(language.get_language("Archive saved to %s.") % (utilities.get_tmpgz()))
    else:
        logger.vlog(language.get_language("Cleaning tmps."))
        utilities.clean_tmps()

############
# Main
#####
if __name__ == "__main__":
    import doctest
    doctest.testmod()
