#!/usr/bin/env python
import glob
import uuid
import json
import os


class RhevCoordinatorRhevh():

    # Containers
    utilities = None
    logger = None
    constants = None
    ssh_client = None
    rhevm = None
    commands = None
    config = None
    language = None

    # Constructor
    def __init__(self, **kwargs):
        self.constants = kwargs['constants']
        self.config = kwargs['config']
        self.logger = kwargs['logger']
        self.utilities = kwargs['utilities']
        self.language = kwargs['language']
        self.ssh_client = kwargs['ssh_client']
        self.rhevm = kwargs['rhevm']

    # Test if a hypervisor is alive
    def test_alive(self, hypervisor):
        ping_test = int(self.utilities.run_command(self.utilities.get_command('ping_command') % (str(hypervisor))))
        if ping_test == 0:
            self.logger.vlog(self.language.get_language("Hypervisor %s is alive.") % (hypervisor))
            return True
        else:
            self.logger.vlog(self.language.get_language("Hypervisor %s is not alive.") % (hypervisor))
            return False

    # Test if a hypervisor can be SSHd into
    def test_ssh(self, hypervisor):
        ssh_conn = self.ssh_client.get_ssh_client_connection(hypervisor)
        if ssh_conn:
            self.logger.vlog(self.language.get_language("Hypervisor %s SSH successful.") % (hypervisor))
            return True
        else:
            self.logger.vlog(self.language.get_language("Hypervisor %s SSH was not successful.") % (hypervisor))
            return False

    # Install/Copy the insights RPM on the hypervisor in /var/tmp
    # Untar insights on the hypervisor
    # This assumes that we have tested and have a valid connection with the specified hypervisor
    def install_hypervisor(self, ip, options=None):
        # check if the local rhevm installation directory/rpm/gz exists
        local_rhevm_installation_exists = self.rhevm.check_rhevm_installation()
        if not local_rhevm_installation_exists:
            self.rhevm.install_rhevm(options)

        # copy the installation directory/rpm/gz over to be run on remote hypervisor
        absolute_insights_path = glob.glob(self.config.get_config_var('local_rhevm_installation'))[0]
        copy_return = self.copy_file_to_hypervisor(ip, absolute_insights_path, self.constants.rhevh_installation_gz)
        if not copy_return:
            self.logger.vlog(self.language.get_language("There was an issue copying Insights on %s") % (ip))
            return False
        else:
            self.logger.vlog(self.language.get_language("Successfully copied Insights on %s") % (ip))

        # untar the scpd gz on the hypervisor
        gz_return = self.untar_hypervisor(ip, options)
        if gz_return:
            self.logger.vlog(self.language.get_language("Successfully untarred Insights on %s") % (ip))
            return True
        else:
            self.logger.vlog(self.language.get_language("There was an issue untarring Insights on %s") % (ip))
            return False

    # Copy the files to the hypervisor
    def copy_file_to_hypervisor(self, hypervisor, source, destination):
        ssh_conn = self.ssh_client.get_ssh_client_connection(hypervisor)
        sftp = ssh_conn.open_sftp()
        try:
            sftp_put = sftp.put(source, destination)
            sftp.close()
            return True
        except Exception as inst:
            return False

    # Untar the insights installation on the hypervisor
    def untar_hypervisor(self, hypervisor, options):
        ssh_command = self.utilities.get_command('untar_hypervisor') % (self.constants.rhevh_installation_dir,
                                                                        self.constants.rhevh_installation_dir,
                                                                        self.constants.rhevh_installation_gz)
        ssh_execution = int(self.ssh_client.run_command_on_client(ssh_command, hypervisor)['stdout'].read())
        if ssh_execution == 0:
            return True
        else:
            return False

    # Run the insights RPM on the hypervisor in /tmp and get STDOUT
    # This assumes that we have tested and have a valid connection with the specified hypervisor
    # This assumes we have copied over the RPM into /tmp
    def run_hypervisor(self, ip, uuid, hypervisor_name, machine_id):

        # for good measures should we check if
        # the remote rhevh installation has been scpd over before running?
        # or should we assume?
        remote_rhevh_installation = self.check_rhevh_installation(ip)

        # if the installation exists then continue
        if remote_rhevh_installation:
            self.logger.vlog(self.language.get_language("Installation exists on RHEV Hypervisor %s(%s), "
                                                        "running Insights.") % (hypervisor_name, ip))

            # Get the insights config, contains uploader.json and sig
            # config['sig'], config['uploader.json']
            config = self.utilities.get_insights_config()
            config['machine-id'] = machine_id
            config = json.dumps(config)

            # Write the temporary configuration file on the hypervisor
            insights_config_file_name = "/var/tmp/%s" % (machine_id)
            insights_config_file = open(insights_config_file_name, "wb")
            insights_config_file.write(config)
            insights_config_file.close()
            self.copy_file_to_hypervisor(ip, insights_config_file_name, self.constants.insights_config_file)
            os.remove(insights_config_file_name)

            # Run Insights
            insights_command = self.utilities.get_command('insights_execution_command') % (self.constants.insights_config_file)
            command_to_run = "cd %s && %s" % (self.constants.rhevh_installation_dir, insights_command)
            ssh_execution = self.ssh_client.run_command_on_client(command_to_run, ip)
            insights_stdout = ssh_execution['stdout'].read()
            return insights_stdout

        else:
            self.logger.vlog(self.language.get_language("Insights could not be found on the RHEV Hypervisor %s(%s).") % (hypervisor_name, ip))
            return False

    # Clean up the hypervisors /tmp from the insights execution
    def clean_hypervisor(self, hypervisor):
        # clean up the remote rhevh installation directodry/gz
        self.logger.vlog(self.language.get_language("Cleaning hypervisor %s") % (hypervisor))

        # TODO: Should this be in the commands.py file?
        hypervisor_command_to_run = "rm -rf %s %s %s ; echo $?" % (self.constants.rhevh_installation_dir,
                                                                   self.constants.rhevh_installation_gz,
                                                                   self.constants.insights_config_file)
        ssh_execution = int(self.ssh_client.run_command_on_client(hypervisor_command_to_run,
                                                                  hypervisor)['stdout'].read())

        if ssh_execution == 0:
            self.logger.vlog(self.language.get_language("Hypervisor %s successfully cleaned.") % (hypervisor))
            return True
        else:
            self.logger.vlog(self.language.get_language("There was an issue cleaning %s.") % (hypervisor))
            return False

    # Check if the RHEVH has received the correct Insights
    def check_rhevh_installation(self, hypervisor):
        ssh_command = self.utilities.get_command('check_rhevh_installation') % (self.constants.rhevh_installation_gz,
                                                                                self.constants.rhevh_installation_dir)
        ssh_execution = int(self.ssh_client.run_command_on_client(ssh_command, hypervisor)['stdout'].read())

        self.logger.vlog(self.language.get_language("Checking if Insights is installed on hypervisor %s") % (hypervisor))

        if ssh_execution == 0:
            self.logger.vlog(self.language.get_language("Insights is installed on hypervisor %s") % (hypervisor))
            return True
        else:
            self.logger.vlog(self.language.get_language("Insights installation not found on hypervisor %s") % (hypervisor))
            return False

    # Generate a machine-id for a hypervisor
    # Uses the machines RHEV UUID from API and the manager system id
    def generate_machine_id(self, rhev_uuid, manager_system_id):
        return str(uuid.uuid5(uuid.UUID(rhev_uuid), manager_system_id))
