#!/usr/bin/env python
import glob
import json
import re
from insights_coordinator_rhev.api import RhevCoordinatorApi
from insights_coordinator_rhev.rhevh import RhevCoordinatorRhevh


class RhevCoordinatorRhevm():

    # Containers
    constants = None
    logger = None
    utilities = None
    api = None
    rhevh = None
    ssh_client = None
    config = None
    language = None
    hypervisor_list = None
    blacklist = None

    # Constructor
    def __init__(self, **kwargs):
        self.constants = kwargs['constants']
        self.logger = kwargs['logger']
        self.ssh_client = kwargs['ssh_client']
        self.utilities = kwargs['utilities']
        self.config = kwargs['config']
        self.language = kwargs['language']
        self.api = RhevCoordinatorApi(constants=self.constants,
                                      config=self.config,
                                      logger=self.logger,
                                      language=self.language)
        self.rhevh = RhevCoordinatorRhevh(constants=self.constants,
                                          config=self.config,
                                          logger=self.logger,
                                          utilities=self.utilities,
                                          language=self.language,
                                          ssh_client=self.ssh_client,
                                          rhevm=self)

    # Setup the Blacklist Containers for Nodes
    def setup_blacklist(self, blacklist):
        self.blacklist = blacklist

    # Test the RHEVM API
    # Returns boolean status connection
    def test_api(self):
        return True if self.api.get_api() else False

    # Install the latest insights RPM on the RHEVM
    # This RPM will be used to distribute to other hypervisors
    def install_rhevm(self, options=None):
        self.logger.vlog(self.language.get_language("Manager does not contain Insights installation. "
                                                    "Attempting to install Insights on manager..."))

        install_cmd = self.utilities.get_command('install_rhevm_auto')
        self.logger.vlog(self.language.get_language("Installing Insights on the manager..."))
        self.utilities.run_command(install_cmd)

    # Check if Insights is on the RHEVM
    def check_rhevm_installation(self):
        insights_installations = glob.glob(self.config.get_config_var('local_rhevm_installation'))
        if insights_installations:
            self.logger.vlog(self.language.get_language("Check for Insights installation on the manager passed."))
            return True
        else:
            self.logger.vlog(self.language.get_language("Check for Insights installation on the manager failed."))
            return False

    # Get the instance of the RHEV Hypervisor
    def get_rhevh(self):
        return self.rhevh

    # Get the manager system ID
    def get_manager_system_id(self):
        return self.utilities.run_command(self.utilities.get_command("get_manager_system_id")).rstrip()

    # Get hypervisors from the rhevm
    def get_hypervisor_list(self):
        if not self.hypervisor_list:
            api_conn = self.api.get_api()
            manager_system_id = self.get_manager_system_id()
            hypervisor_list = [[ahypervisor.get_address(), ahypervisor.get_id(), ahypervisor.get_name(), self.rhevh.generate_machine_id(ahypervisor.get_id(), manager_system_id)] for ahypervisor in api_conn.hosts.list()]
            hypervisor_list = self.test_hypervisor_list(hypervisor_list)
            self.hypervisor_list = hypervisor_list
            self.logger.vlog(self.language.get_language("Gathering hypervisor list from manager..."))
            self.logger.vlog(self.language.get_language("Found hypervisors:"))
            self.logger.vlog(hypervisor_list)
        return self.hypervisor_list

    # Test a list of hypervisors
    def test_hypervisor_list(self, hypervisor_list):
        tested_list = []
        if hypervisor_list:
            for hypervisor in hypervisor_list:
                parsed_hypervisor = {'ip': hypervisor[0], 'id': hypervisor[1], 'name': hypervisor[2], 'system_id': hypervisor[3]}
                parsed_hypervisor['blacklisted'] = True if (self.blacklist is not False and len(self.blacklist) > 0 and len([keyword for keyword in self.blacklist if re.match(keyword, hypervisor[2])]) > 0) else False
                parsed_hypervisor['is_alive'] = self.rhevh.test_alive(hypervisor[0])
                parsed_hypervisor['can_ssh'] = self.rhevh.test_ssh(hypervisor[0])
                tested_list.append(parsed_hypervisor)
        return tested_list

    # Print the hypervisor list
    def print_hypervisor_list(self, hypervisor_list):
        if hypervisor_list:
            hypervisor_keys = hypervisor_list[0].keys()
            self.logger.vlog(self.language.get_language("========================= Hypervisor List "
                                                        "========================="))
            self.logger.vlog(("%-40s" * len(hypervisor_keys)) % tuple(hypervisor_keys))
            self.logger.vlog("-------------------------------------------------------------")
            for hypervisor in hypervisor_list:
                self.logger.vlog(("%-40s" * len(hypervisor)) % tuple(hypervisor.values()))
            self.logger.vlog("=============================================================")

    # Collect information for the manager from the API
    def collect_manager_information(self, options=None):
        if not self.utilities.get_tmpdir():
            self.utilities.setup_tmpdir()
        manager_information = {}

        # Get all metadata from API
        self.logger.vlog(self.language.get_language("Using the RHEVM API to collect manager information..."))
        self.logger.vlog(self.language.get_language("Using Manager options:"))
        self.logger.vlog(options)
        manager_information = self.api.get_manager_data_from_api(options)

        # Get API Version
        api_version = self.get_rhev_api_version()
        self.logger.vlog(self.language.get_language("Getting RHEV API Version..."))
        self.logger.vlog(api_version)
        manager_information["api_version"] = api_version

        # Get Manager OS Version
        os_version = self.get_rhev_os_version()
        self.logger.vlog(self.language.get_language("Getting RHEV OS Version..."))
        self.logger.vlog(os_version)
        manager_information["rhev_version"] = os_version

        # Get Insights Version
        insights_version = self.get_insights_version()
        self.logger.vlog(self.language.get_language("Getting Insights Version..."))
        self.logger.vlog(insights_version)
        manager_information["insights_version"] = insights_version

        # Get Coordinator Version
        coordinator_version = self.get_coordinator_version()
        self.logger.vlog(self.language.get_language("Getting Coordinator Version..."))
        self.logger.vlog(coordinator_version)
        manager_information["coordinator_version"] = coordinator_version

        # Setup the systems relationships for the mananger and hypervisors
        manager_information["systems"] = self.get_systems()

        # Setup the system ID
        # For the simplicity of the engine, this is passed through as "system_id" even though it is cluster-id on the actual machine
        # This is still a UUID for the entire cluster, its not the system_id or machine-id for the Manager
        manager_information["system_id"] = self.utilities.get_cluster_id()

        # Get the manager display name
        manager_information["display_name"] = self.get_display_name()

        # Add the product information
        # This allows the engine to intelligently decide what to do for the upload
        manager_information["product"] = self.get_product_code()

        # Write the log
        log_information_string = json.dumps(manager_information, default=self.utilities.serialize_default)
        self.logger.vlog(self.language.get_language("Writing manager information to temporary directory..."))
        self.utilities.write_file(log_information_string, self.constants.manager_metadata_filename)

    # Get system relationships for hypervisors and manager
    def get_systems(self):
        systems = []
        manager_system_id = self.get_manager_system_id()
        for ahypervisor in self.get_hypervisor_list():
            thehypervisor = {}
            thehypervisor['type'] = 'Hypervisor'
            thehypervisor['product'] = self.constants.product_code
            thehypervisor['system_id'] = ahypervisor['system_id']
            thehypervisor['display_name'] = ahypervisor['name']
            thehypervisor['ip'] = ahypervisor['ip']
            thehypervisor['links'] = [{'type': 'Manager', 'system_id': manager_system_id}]
            thehypervisor['blacklisted'] = ahypervisor['blacklisted']
            systems.append(thehypervisor)
        manager = {}
        manager['type'] = 'Manager'
        manager['product'] = self.constants.product_code
        manager['system_id'] = manager_system_id
        manager['display_name'] = self.get_display_name()
        manager['links'] = []
        manager['hypervisor'] = False
        systems.append(manager)
        return systems

    # Get the manager name / ID
    def get_display_name(self):
        return self.utilities.run_command(self.utilities.get_command("get_manager_name")).rstrip()

    # Get the manager system ID
    def get_manager_system_id(self):
        return self.utilities.run_command(self.utilities.get_command("get_manager_system_id")).rstrip()

    # Get the RHEV API Version for Metadata
    def get_rhev_api_version(self):
        api_conn = self.api.get_api()
        return api_conn.get_product_info().get_full_version()

    # Get the RHEV OS Version for Metadata
    def get_rhev_os_version(self):
        return self.utilities.run_command(self.utilities.get_command('get_os_version')).rstrip()

    # Get the Insights Version for Metadata
    def get_insights_version(self):
        return self.utilities.run_command(self.utilities.get_command('get_insights_version')).rstrip()

    # Get the Coordinator Version for Metadata
    def get_coordinator_version(self):
        return self.constants.version

    # Get the product code for this coordinator
    def get_product_code(self):
        return self.constants.product_code

    # Run Insights on the Manager
    def run_insights_on_manager(self):

        # Run insights
        self.logger.vlog(self.language.get_language('Running Insights on manager'))
        command_to_run = self.utilities.get_command('run_insights_on_manager')
        stdout = self.utilities.run_command(command_to_run)
        self.utilities.write_archive(stdout, 'insights-coordinator-rhevm')
